const { Op } = require('sequelize');
const builder = require('../audit.query');

describe('Audit query builder', () => {
    describe('_buildDates', () => {
        it('should return default val if dates not exists', () => {
            const result = builder._buildDates({
                from: '',
                to: ''
            });

            expect(result).toEqual({ createdAt: {} });
        });

        it('should return correct dates', () => {
            const time = new Date();
            const result = builder._buildDates({
                from: time.getTime(),
                to: time.setSeconds(1)
            });

            expect(typeof result.createdAt[Op.gte]).toEqual('number');
            expect(typeof result.createdAt[Op.lte]).toEqual('number');
        });
    });

    describe('_buildSearchFilters', () => {
        it('should return correct filters', () => {
            const result = builder._buildSearchFilters({
                userName: ['testName', 'testName2'],
                userId: 'testUserId'
            });

            expect(result.userId).toEqual('testUserId');
            expect(result.userName[Op.or]).toEqual(['testName', 'testName2']);
        });

        it('should return correct filters', () => {
            const result = builder._buildSearchFilters({
                userName: ['testName', 'testName2'],
                userId: 'testUserId',
                search: 'text'
            });

            expect(result.userId).toEqual('testUserId');
            expect(result.userName[Op.or]).toEqual(['testName', 'testName2']);
        });

        it('should handle flags query', () => {
            const result = builder._buildSearchFilters({
                showTriggers: 'true'
            });

            expect(result.action).toEqual('receiveWebhook');
        });
    });

    describe('_buildSorting', () => {
        it('should return default when sort field not support', () => {
            const result = builder._buildSorting();

            expect(result).toEqual({});
        });

        it('should return correct sort', () => {
            const result = builder._buildSorting({
                sortField: 'createdAt',
                sortType: 'DESC'
            });

            expect(result).toEqual({ order: [['createdAt', 'DESC']] });
        });
    });
});
