/* eslint class-methods-use-this: 0 */
const _ = require('lodash');
const { Op } = require('sequelize');

const models = require('../models');

let Audit = {};

class AuditDal {

    async init(needCreateDb) {
        if (needCreateDb) {
            await models.createDatabaseIfNotExists();
        }

        ({ Audit } = models.getInstance());
    }

    store(audit) {
        return Audit.create(audit);
    }

    getAudit(query) {
        return Audit.findAndCountAll(query);
    }

    async getUserNames(accountId) {
        const data = await Audit.aggregate('userName', 'DISTINCT', {
            plain: false,
            where: { accountId },
        });
        return data.map(item => item.DISTINCT);
    }

    getEntitiesList(accountId, opts) {
        const where = { accountId };

        if (_.get(opts, 'showTriggers')) {
            where.action = 'receiveWebhook';
        } else {
            where.action = { [Op.ne]: 'receiveWebhook' };
        }

        return Audit.findAll({
            where,
            attributes: ['entity', 'action'],
            distinct: true,
            raw: true,
        });
    }
}

module.exports = new AuditDal();
