const fs = require('fs');
const path = require('path');
const Sequelize = require('sequelize');

const basename = path.basename(module.filename);
const config = require('../config');

const { pgConfig } = config;


const createDatabaseIfNotExists = async () => {
    const sequelize = new Sequelize('postgres', pgConfig.user, pgConfig.password, {
        dialect: 'postgres',
        host: pgConfig.host,
    });

    try {
        await sequelize.query(`CREATE DATABASE ${pgConfig.database};`);
    } catch (e) {
        console.log(e);
    }

    return sequelize.close();
};

const getInstance = () => {
    const db = {};

    const sequelize = new Sequelize(pgConfig.database, pgConfig.user, pgConfig.password, {
        host: pgConfig.host,
        dialect: 'postgres',
        logging: false,
        pool: {
            max: pgConfig.max,
            min: 0,
            acquire: 30000,
            idle: pgConfig.idleTimeoutMillis,
        },

    });

    fs
        .readdirSync(__dirname)
        .filter((file) => {
            return (file.indexOf('.') !== 0) && (file !== basename) && (file.slice(-3) === '.js');
        })
        .forEach((file) => {
            const model = sequelize.import(path.join(__dirname, file));
            db[model.name] = model;
        });

    Object.keys(db)
        .forEach((modelName) => {
            if (db[modelName].associate) {
                db[modelName].associate(db);
            }
        });


    Object.keys(db)
        .forEach((modelName) => {
            db[modelName].sync({ alter: true });
        });

    db.sequelize = sequelize;
    db.Sequelize = Sequelize;

    return db;
};


module.exports = {
    getInstance,
    createDatabaseIfNotExists,
};
